package main

import (
	"bufio"
	"bytes"
	"fmt"
	"io"
	"strings"
)

func main() {
	// string Reader 读数据
	reader := strings.NewReader("hello\n world")
	bs := make([]byte, 2)
	for {
		n, err := reader.Read(bs)
		if err != nil {
			if err != io.EOF {
				fmt.Println(err)
			}
			break
		}
		fmt.Print(string(bs[:n]))
	}
	fmt.Println()

	// 文件指针还原
	_, _ = reader.Seek(0, 0)

	// 使用 bufio 带缓冲的读
	scanner := bufio.NewScanner(reader)
	for scanner.Scan() {
		fmt.Println(scanner.Text())
	}

	// string Builder 写数据，如字符串拼接，效率比较高
	//var builder strings.Builder
	builder := strings.Builder{}
	builder.Write([]byte("hello strings builder"))
	builder.WriteString("hello builder")
	fmt.Println(builder.String(), builder.Len())

	// https://pkg.go.dev/bytes
	// bytes.Reader
	breader := bytes.NewReader([]byte("hello bytes Reader"))
	bs = make([]byte, 5)
	for {
		n, err := breader.Read(bs)
		if err != nil {
			if err != io.EOF {
				fmt.Println(err)
			}
			break
		}
		fmt.Print(n, " ", string(bs[:n]))
	}
	fmt.Println()

	// bytes.Buffer
	// buffer := bytes.NewBuffer([]bytes(""))
	buffer := bytes.NewBufferString("hello bytes.Buffer")
	buffer.Write([]byte("\nxianbin"))
	buffer.WriteString("\nhave a nice day!")
	fmt.Println(buffer.String())

	bs = make([]byte, 4)
	fmt.Println(buffer.Read(bs))
	fmt.Println(string(bs))

	// 读到指定分隔符，包含分隔符 delim
	line, err := buffer.ReadString('\n')
	fmt.Println(line, err)

	bs2, err := buffer.ReadBytes('\n')
	fmt.Println(string(bs2), err)
}
